package favi;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.MissingResourceException;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;

import jGDK.ErrManager;
import jGDK.Framstick;
import jGDK.GenotypeParser;
import jGDK.I18N;
import jGDK.Library;
import jGDK.Message;

/**
 * <p>Title: Favi - Framsticks applet viewer</p>
 * <p>Description: Applet made to view Framsticks from genotype in the html browser.</p>
 * @author Damian Jarz?bski
 * @version 2.0
 */
public class Favi extends JApplet implements ActionListener
{
  private Vector wynik;
  private Button startButton = null;
  private String buttonName = null;
  Tazo tazo = null;
  Symbol symbol = null;
  Framstick fram = null;
  JTextPane jTP = null;
  SimpleAttributeSet[] attrs = null;
  JSplitPane jSP1 = null;
  JScrollPane jScrollP = null;
  URL codeBase;
  JFrame infoFrame = null;
  JFrame aboutFrame = null;
  JMenuBar jMB = null;
  JMenuItem infoMenu = null;
  JMenuItem aboutMenu = null;
  F0Viewer editor = null;
  
  /**
   * initing jGDK
   */
  private void initJGDK()
  {
    ErrManager err = new ErrManager();
    Library library = null;
    append("Favi", I18N.getMessage("InitJGDK"), ErrManager.ER_MESSAGE);
    try
    {
      library = new Library();
    } catch(Exception ex)
    {
      System.err.println(I18N.getMessage("Error") + ex.getLocalizedMessage());
      append("Init", ex.getLocalizedMessage(), ErrManager.ER_ERROR);
    }
    Framstick.library = library;
    GenotypeParser.library = library;
    Symbol.library = library;
    for (int x = 0; x < ErrManager.msg.size(); x++)
    {
      System.out.println(((Message)ErrManager.msg.get(x)).description);
      append("Init", ((Message)ErrManager.msg.get(x)).description, ((Message)ErrManager.msg.get(x)).status);
    }
    ErrManager.msg.clear();
  }
  /**
   *  creating messages view.
   */
  private void makePanel()
  {
    if (jTP == null)  jTP = new JTextPane();
    attrs = new SimpleAttributeSet[4];
    attrs[ErrManager.ER_MESSAGE] = new SimpleAttributeSet();
    StyleConstants.setFontFamily(attrs[ErrManager.ER_MESSAGE], "Arial");
    StyleConstants.setFontSize(attrs[ErrManager.ER_MESSAGE], 12);
    StyleConstants.setBold(attrs[ErrManager.ER_MESSAGE], true);
    
    attrs[ErrManager.ER_WARNING] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setItalic(attrs[ErrManager.ER_WARNING], true);
    StyleConstants.setForeground(attrs[ErrManager.ER_WARNING], Color.orange);
    
    attrs[ErrManager.ER_HINT] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setForeground(attrs[ErrManager.ER_HINT], Color.yellow);
    
    attrs[ErrManager.ER_ERROR] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setForeground(attrs[ErrManager.ER_ERROR], new Color(164, 20, 70));
    jScrollP = new JScrollPane();
    jScrollP.setMinimumSize(new Dimension(50, 21));
    //    jScrollP.setPreferredSize(new Dimension(400, 300));
    jScrollP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    jTP.setBackground(new Color(100, 100, 250));
    jTP.setPreferredSize(new Dimension(200, 300));
    jTP.setToolTipText("");
    jTP.setEditable(false);
    jTP.setEditorKit(new StyledEditorKit());
    jScrollP.getViewport().add(jTP, null);
  }
  /**
   * creating information dialog
   */
  private void makeInfo()
  {
    jMB = new JMenuBar();
    infoMenu = new JMenuItem(I18N.getMessage("Information"));
    infoMenu.addActionListener(this);
    jMB.add(infoMenu);
    aboutMenu = new JMenuItem(I18N.getMessage("About"));
    aboutMenu.addActionListener(this);
    jMB.add(aboutMenu);
    if (infoFrame == null)
    {
      infoFrame = new JFrame(I18N.getMessage("Information"));
      infoFrame.getContentPane().add(jScrollP);
      infoFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      infoFrame.pack();
      infoFrame.setSize(400, 200);
      //      infoFrame.setLocation(100, 100);
    }
    if (aboutFrame == null)
    {
      aboutFrame = new JFrame(I18N.getMessage("About"));
      JTextPane tp=new JTextPane();
      tp.setEditable(false);
      String[][] s = getParameterInfo();
      String ss=I18N.getMessage("FaviTitle")+"\n(C) 2003, 2004\n\n";
      ss+=I18N.getMessage("authors") + " Damian Jarzebski, Piotr Jankowski, Marcin Jurkiewicz, Cezary Dowhan\n";
      ss+=I18N.getMessage("mentor") + " Maciej Komosinski, Szymon Ulatowski\n\n\n";
      ss+=I18N.getMessage("Params") + ":\n";
      for (int i = 0; i < s.length; i++)   ss+="   "+s[i][0] + " - " + s[i][1] + " - " + s[i][2]+"\n";
      tp.setText(ss);
      aboutFrame.getContentPane().add(tp);
      aboutFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      aboutFrame.setResizable(false);
      aboutFrame.pack();
    }
  }
  /**
   * downloading page with specified genotype.
   */
  String[] downloadGen(String address)
  {
    if (address == null)
    {
      System.err.println("The download address is null!");
      return null;
    }
    BufferedReader in = null;
    String input = new String("");
    URL u = null;
    
    // jak uzyskac url z nazwy - wersja krotka
    try
    {u=new URL(address);} // url moze byc podany bezposrednio (http:// file://)
    catch (MalformedURLException e)
    {
      try
      {u=new URL(getCodeBase(),address);} // albo wzgledem codebase
      catch (MalformedURLException e2)
      {System.err.println(e2.getLocalizedMessage());}
    }
    
/*
 //         ...wersja dluga ale za to niedzialajaca
 // 	    (ciekawe ze w kazdym miejscu gdzie cos wczytuja jest to zrobione podobnie
 //	    ale jednak troche inaczej. moze uzyli do tego algorytmow genetycznych?)
 
                if (!address.startsWith("http")) {
                        if (!address.startsWith("www")) {
                                try {
                                        u = new URL(this.getCodeBase().getProtocol(), "", this.getCodeBase().getFile() + address);
                                }
                                catch (MalformedURLException e) {
                                        System.err.println(e.getLocalizedMessage());
                                }
                        }
                        else {
                                address = "http://" + address;
                                try {
                                        u = new URL(address);
                                }
                                catch (MalformedURLException e) {
                                        System.err.println(e.getLocalizedMessage());
                                }
                        }
                }
                else
                        try {
                                u = new URL(address);
                        }
                        catch (MalformedURLException e) {
                                System.err.println(e.getLocalizedMessage());
                        }
 */
    System.out.println("URL:"+u+"\nHOST:" + u.getHost() + "\nFILE:" + u.getFile());
    URLConnection uc = null;
    try
    {
      uc = u.openConnection();
    }
    catch (IOException e2)
    {
      System.err.println(e2.getLocalizedMessage());
    }
    try
    {
      try
      {
        in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
      }
      catch (IOException e1)
      {
        System.out.println(I18N.getMessage("Error") + ": " + I18N.getMessage("DownloadFile"));
        System.out.println(e1.getLocalizedMessage());
        append("Init", I18N.getMessage("DownloadFile"), ErrManager.ER_ERROR);
      }
    }
    catch(java.security.AccessControlException e2)
    {
      System.out.println(I18N.getMessage("Error") + ": " + I18N.getMessage("DownloadFile"));
      System.out.println(e2.getLocalizedMessage());
      append("Init", I18N.getMessage("DownloadFile"), ErrManager.ER_ERROR);
    }
    if (in != null)
      try
      {
        String inputLine;
        while ((inputLine = in.readLine()) != null)
        {
          input += inputLine + "\n";
        };
        in.close();
      }
      catch (IOException expt)
      {
        System.err.println(I18N.getMessage("Error") + ": " + expt.getLocalizedMessage());
        append("Init", expt.getLocalizedMessage(), ErrManager.ER_ERROR);
      };
      //System.out.println("INPUT:"+input);
      if (input.equalsIgnoreCase(""))
      {
        return null;
      }
      else
      {
        String[] tab =
        {
          I18N.getMessage("unknownName"),
          I18N.getMessage("unknownAuthor"),
          I18N.getMessage("noInfo"),
          "//0" 
          };
          StringTokenizer st = new StringTokenizer(input, "<>");
          String token;
          while (st.hasMoreTokens())
          {
            token = st.nextToken();
            if (token.equalsIgnoreCase("FRED-NAME"))
              tab[0] = st.nextToken();
            else if (token.equalsIgnoreCase("FRED-AUTHOR"))
              tab[1] = st.nextToken();
            else if (token.equalsIgnoreCase("FRED-INFO"))
              tab[2] = st.nextToken();
            else if (token.equalsIgnoreCase("FRED-NOTES"))
              tab[2] += "\n\nNotes:\n"+st.nextToken();
            else if (token.toUpperCase().startsWith("!--FRED_GEN"))
            {
              tab[3] = token.replace('~', '\n').substring(11, token.length() - 2);
            }
          }
          return tab;
      }
  }
  /**
   * creating all visualization
   *
   */
  private void createAll()
  {
    jSP1 = new JSplitPane();
    makePanel();
    initJGDK();
    makeInfo();
    //Get the applet parameters.
    String at = getParameter("DB_URL");
    //System.out.println("Genotype link: " + at);
    
    String[] tab = downloadGen(at);
    fram = new Framstick();
    if (tab != null)
    {
      fram.name = tab[0];
      fram.author = tab[1];
      fram.info = tab[2];
      fram.genotype = tab[3];
      fram.rebuildModel();
      append("Favi", I18N.getMessage("LoadModel"), ErrManager.ER_MESSAGE);
      append(I18N.getMessage("NAME"), fram.name, ErrManager.ER_HINT);
      append(I18N.getMessage("AUTHOR"), fram.author, ErrManager.ER_HINT);
      append(I18N.getMessage("INFO"), fram.info, ErrManager.ER_HINT);
      for (int x = 0; x < ErrManager.msg.size(); x++)
      {
        System.out.println(((Message)ErrManager.msg.get(x)).description);
        append("Parser", ((Message)ErrManager.msg.get(x)).description, ((Message)ErrManager.msg.get(x)).status);
      }
      ErrManager.msg.clear();
      
      tazo = new Tazo(fram);
      symbol = new Symbol(fram);
      
      jSP1.add(tazo, JSplitPane.TOP);
      jSP1.add(symbol, JSplitPane.BOTTOM);
      jSP1.setContinuousLayout(true);
      //jSP1.setResizeWeight(0.5); jre 1.3 required
      JMenuItem m = new JMenuItem(I18N.getMessage("f0Genotype"));
      m.setActionCommand("geno");
      m.addActionListener(this);
      jMB.add(m,1); //index 1 - after "information"
    }
    else
    {
      at = getParameter("genotype");
      if (at != null)
      {
        at.replace('~', '\n');
        fram.genotype = at;
        at = getParameter("name");
        fram.name = (at != null) ? at : I18N.getMessage("Unknown");
        at = getParameter("author");
        fram.author = (at != null) ? at : I18N.getMessage("Unknown");
        at = getParameter("info");
        fram.info = (at != null) ? at : "-";
        fram.rebuildModel();
        append("Favi", I18N.getMessage("LoadModel"), ErrManager.ER_MESSAGE);
        append(I18N.getMessage("NAME"), fram.name, ErrManager.ER_HINT);
        append(I18N.getMessage("AUTHOR"), fram.author, ErrManager.ER_HINT);
        append(I18N.getMessage("INFO"), fram.info, ErrManager.ER_HINT);
        for (int x = 0; x < ErrManager.msg.size(); x++)
        {
          System.out.println(((Message)ErrManager.msg.get(x)).description);
          append("Parser", ((Message)ErrManager.msg.get(x)).description, ((Message)ErrManager.msg.get(x)).status);
        }
        ErrManager.msg.clear();
        
        tazo = new Tazo(fram);
        symbol = new Symbol(fram);
        
        jSP1.add(tazo, JSplitPane.TOP);
        jSP1.add(symbol, JSplitPane.BOTTOM);
        jSP1.setContinuousLayout(true);
        //jSP1.setResizeWeight(0.5);
        JMenuItem m = new JMenuItem(I18N.getMessage("f0Genotype"));
        m.setActionCommand("geno");
        m.addActionListener(this);
        jMB.add(m,1); //index 1 - after "information"
      }
      else
      {
        JLabel l = new JLabel(I18N.getMessage("DownloadFile"));
        jSP1.add(l, JSplitPane.TOP);
        jSP1.add(l, JSplitPane.BOTTOM);
        jSP1.setContinuousLayout(true);
      }
    }
  }
  /**
   * making labels if version is below 1.2.
   * @param cp
   * @param ver
   */
  private void makeLabels(Container cp, String ver)
  {
    cp.setLayout(new FlowLayout());
    Label l = new Label();
    l.setText("You have java version " + ver);
    cp.add(l);
    l = new Label();
    l.setText("You should have java version 1.2 or newer");
    cp.add(l);
    l = new Label();
    l.setText("You can download it from:");
    cp.add(l);
    l = new Label();
    l.setText("www.java.sun.com");
    cp.add(l);
  }
  /**
   * main function starting applet. It checks java version, shows panels or only the button
   * (depends of applet params).
   */
  public void init()
  {
    String ver = System.getProperty("java.version");
    float zm = (Float.valueOf(ver.substring(0, 3))).floatValue();
    if (zm < 1.2)
    {
      if (this.getSize().width > 100)
      {
        makeLabels(this.getContentPane(), ver);
      }
      else
      {
        JFrame f = new JFrame("Error");
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        makeLabels(f.getContentPane(), ver);
        f.setResizable(false);
        f.pack();
        f.show();
      }
      return;
    }
    
    attrs = new SimpleAttributeSet[4];
    attrs[ErrManager.ER_MESSAGE] = new SimpleAttributeSet();
    StyleConstants.setFontFamily(attrs[ErrManager.ER_MESSAGE], "Arial");
    StyleConstants.setFontSize(attrs[ErrManager.ER_MESSAGE], 12);
    StyleConstants.setBold(attrs[ErrManager.ER_MESSAGE], true);
    
    attrs[ErrManager.ER_WARNING] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setItalic(attrs[ErrManager.ER_WARNING], true);
    StyleConstants.setForeground(attrs[ErrManager.ER_WARNING], Color.orange);
    
    attrs[ErrManager.ER_HINT] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setForeground(attrs[ErrManager.ER_HINT], Color.yellow);
    
    attrs[ErrManager.ER_ERROR] = new SimpleAttributeSet(attrs[ErrManager.ER_MESSAGE]);
    StyleConstants.setForeground(attrs[ErrManager.ER_ERROR], new Color(164, 20, 70));
    
    String language = null;
    String country = null;
    language = getParameter("language");
    country = getParameter("country");
    //    System.out.println(language);
    jTP = new JTextPane();
    try
    {
      I18N i18N = new I18N(language, (country != null) ? country : "");
    }
    catch (MissingResourceException e)
    {
      append("Favi", "There was no language file for your country - using US english.", ErrManager.ER_HINT);
      I18N i18N = new I18N("en", "US");
    }
    
    String buttonName = getParameter("button");
    if ((buttonName == null) || (buttonName.equalsIgnoreCase("")))
    {
      createAll();
      this.getContentPane().setLayout(new BorderLayout());
      this.getContentPane().add(jMB, BorderLayout.NORTH);
      this.getContentPane().add(jSP1, BorderLayout.CENTER);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      this.getContentPane().setBounds(d.width/4, d.height/4, d.width*2/4, d.height*2/4);
      jSP1.setDividerLocation(0.7);
    }
    else
    {
      startButton = new Button(buttonName);
      startButton.addActionListener(this);
      this.getContentPane().add(startButton);
    }
  }
  
  /**
   * appending to existing messages.
   * @param name
   * @param text of message
   * @param attr - color of message
   */
  public void append(String name, String text, int attr)
  {
    if (!((attr >= 0) && (attr <= 3)))  attr = 0;
    
    try
    {
      jTP.getStyledDocument().insertString(jTP.getStyledDocument().getLength(), name + ": " + text + "\n", attrs[attr]);
    }
    catch (BadLocationException ble)
    {
      System.err.println(I18N.getMessage("Error") + ": " + I18N.getMessage("CouldInsert"));
    }
  }
  
  /** (non-Javadoc)
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource().equals(startButton))
    {
      String butit=startButton.getLabel();
      startButton.setLabel(I18N.getMessage("Loading"));
      startButton.setEnabled(false);
      final JFrame f = new JFrame();
      f.setTitle(I18N.getMessage("FaviTitle"));
      createAll();
      f.setJMenuBar(jMB);
      f.getContentPane().add(jSP1);
      if (fram!=null) f.setTitle("Favi 2.0 - "+fram.name); // I18N.getMessage("FaviTitle"));
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      f.setBounds(d.width/4, d.height/4, d.width*2/4, d.height*2/4);
      f.addWindowListener(new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
        {
          if (f != null)
          {
            if(editor!=null)
            {
              editor.dispose();
              editor=null;
            }
            if (jMB != null)
              f.remove(jMB);
            if (jSP1 != null)
            {
              if (tazo != null)
                jSP1.remove(tazo);
              if (symbol != null)
                jSP1.remove(symbol);
              f.remove(jSP1);
            }
          }
          wynik = null;
          //startButton = null;
          //buttonName = null;
          tazo = null;
          symbol = null;
          fram = null;
          //jTP = null;
          //attrs = null;
          jSP1 = null;
          jScrollP = null;
          codeBase = null;
          if(infoFrame!=null)
          {
            infoFrame.dispose();
            infoFrame = null;
          }
          if(aboutFrame!=null)
          {
            aboutFrame.dispose();
            aboutFrame = null;
          }
          jMB = null;
          infoMenu = null;
          aboutMenu = null;
          if (f != null) f.dispose();
          startButton.setEnabled(true);
        }
        
      });
      f.show();
      jSP1.setDividerLocation(0.7);
      startButton.setLabel(butit);
    }
    if (e.getSource().equals(infoMenu))
    {
      infoFrame.show();
    }
    if (e.getSource().equals(aboutMenu))
    {
      aboutFrame.show();
    }
    if (e.getActionCommand().equalsIgnoreCase("geno"))
    {
      if (editor == null)
      {
        editor = new F0Viewer(fram);
        editor.initDocument();
        editor.setSize(350, 400);
      }
      editor.show();
    }
  }
  
  public void start()
  {}
  
  public void stop()
  {}
  
  /** (non-Javadoc)
   * @see java.applet.Applet#getAppletInfo()
   */
  public String getAppletInfo()
  {
    return I18N.getMessage("InfoTitle") + "\n" + I18N.getMessage("InfoAuthor") + "\n" + I18N.getMessage("InfoInfo");
  }
  
  /** (non-Javadoc)
   * @see java.applet.Applet#getParameterInfo()
   */
  public String[][] getParameterInfo()
  {
    String[][] info =
    {
      { "DB_URL", I18N.getMessage("string"), I18N.getMessage("FramURL")},
      {
        "genotype", I18N.getMessage("string"), I18N.getMessage("FramGen")
      },
      {
        "name", I18N.getMessage("string"), I18N.getMessage("FramName") + " " + I18N.getMessage("Unknown")
      },
      {
        "author", I18N.getMessage("string"), I18N.getMessage("AuthorsName")
      },
      {
        "info", I18N.getMessage("string"), I18N.getMessage("ParamInfo")
      },
      {
        "language", I18N.getMessage("string"), I18N.getMessage("ParamLang")
      },
      {
        "country", I18N.getMessage("string"), I18N.getMessage("ParamCountry")
      }
    };
    return info;
  }
  /**
   * geting full url to the filename.
   * @param filename
   * @return
   */
  
  // ech... pewnie zamiast tego wystarczy standardowa funkcja getResource
  // ale juz nie chce mi sie sprawdzac :-|
  
  // ech... a ja znow zobaczylem dejavu, po raz kolejny widze ten sam fragment
  // kodu w nastepnym pliku ;(
  protected URL getURL(String filename)
  {
    URL url = null;
    if (codeBase == null)
    {
      codeBase = getCodeBase();
    }
    
    try
    {
      url = new URL(codeBase, filename);
    }
    catch (java.net.MalformedURLException e)
    {
      System.err.println(e.getLocalizedMessage());
      return null;
    }
    
    return url;
  }
}
