package fred;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;

import jGDK.Framstick;
import jGDK.FramstickListener;
import jGDK.FramstickEvent;

/**
 * Class responsible for network visualisation ion the panel in body window.
 */
public class NeuroPanel extends JPanel implements MouseListener, MouseMotionListener 
{
	static int[] standardNeuron = {1,4,70,50,50,50,20,80,20,20,50,50};
	MyFramstickListener mFL=new MyFramstickListener();

	Framstick fram;
	float zoom = 0.5f;
	int oldHeight = 0;	//stare rozmiary okna
	Dimension d;		//zmienna do rozmiarow okna
	int liczbaNeuronow = 0;

	/**
	 *Creates simple empty panel.
	 */
	NeuroPanel(Framstick model)
	{
		fram=model;
		d = getSize();
		fram.addFramstickListener(mFL);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}

	/**
	 * Repaints the panel.
	 */
	public void paint(Graphics g)
	{
		update(g);
	}

	/**
	 * Updates the content of the panel.
 	 */
	public void update(Graphics g)
	{
		d = getSize();									
		g.setColor(new Color(0xaaaaaa));
		g.fillRect(0,0,50,d.height);				
		g.setColor(Color.black);
		g.drawLine(0,0,d.width-1,0);
		g.drawLine(d.width-1,0,d.width-1,d.height-1);
		g.drawLine(d.width-1,d.height-1,0,d.height-1);
		g.drawLine(0,d.height-1,0,0);
		if (d.height!=oldHeight) oldHeight = d.height; 
		setZoomValue();
		drawNetwork(g);
	}

	/**
	 * Draws single neuron symbol.
	 */
	public void drawNeuron(Graphics g, float x, float y, String typ)
	{
		int[] scheme = (int[]) Framstick.library.getRecord("neuro",typ).getInfoAbout("symbol");
		if (scheme == null) scheme = standardNeuron;
		int point1x, point1y, point2x, point2y, position = 0;
		int noOfBlocks = 0;		//liczba ciaglych obszarow w definicji
		int noOfLines = 0;		//liczba linii w kolejnym obszarze
		noOfBlocks = scheme[position++];//liczba ciaglych obszarow
		for(int i=0; i<noOfBlocks; i++)//dla kazdego bloku
		{
			noOfLines = scheme[position++];//liczba linii w obszarze
			point2x = scheme[position++];
			point2y = scheme[position++];
			for(int j=0; j<noOfLines; j++)//dla kazdej linii w bloku
			{
				point1x = point2x;
				point1y = point2y;
				point2x = scheme[position++];
				point2y = scheme[position++];
				g.drawLine((int)((point1x+x*100)*zoom),(int)((point1y+y*100)*zoom),(int)((point2x+x*100)*zoom),(int)((point2y+y*100)*zoom));
			}
		}
	}

	/** 
	 * Draws whole network - a set of neurons positioned in a column.
	 */
	public void drawNetwork(Graphics g)
	{
		int licznik = 0;
		for(int i=0; i<fram.getNeuroCount(); i++)
		{
			if(fram.elementWeakSelected("n",i))// || fram.elementStrongSelected("n",i))
				drawNeuron(g, 0, licznik++, fram.getNeuroProp(i,"d").toString());
		}
	}

	/** 
	 * Scales the scene fitting it to the panel and window size.
 	 */
	public void setZoomValue()
	{
		this.liczbaNeuronow = 0;
		for(int i=0; i<fram.getNeuroCount(); i++)
		{
			if(fram.elementWeakSelected("n",i))// || fram.elementStrongSelected("n",i))	
				this.liczbaNeuronow++;
		}
		float wysokosc = (float)oldHeight;
		float liczba = (float)liczbaNeuronow;
		try	{ this.zoom = wysokosc/(liczba*100); }
		catch(Exception e) { this.zoom = 0.5f; }
		if(this.zoom>0.5f) this.zoom = 0.5f;
	}

	/**
	 * Enables marking neurons shown on the panel.
	 */
	public void markNeuronAt(int y)
	{
		int numer = (int)Math.floor(y / (100*zoom));
		int ile = -1;
		for(int i=0; i<fram.getNeuroCount(); i++)
		{
			if(fram.elementWeakSelected("n",i))// || fram.elementStrongSelected("n",i))	
				ile++;
			if(ile==numer) 
			{
				ile=i;
				break;
			}
		}
	}

	/**
	 * Enables deleting neurons shown on the panel by right clicking on it.
	 */
	public void delNeuronAt(int y)
	{
		int numer = (int)Math.floor(y / (100*zoom));
		int ile = -1;
		for(int i=0; i<fram.getNeuroCount(); i++)
		{
			if(fram.elementWeakSelected("n",i))// || fram.elementStrongSelected("n",i))	
				ile++;
			if(ile==numer) 
			{
				ile=i;
				break;
			}
		}
		fram.removeNeuron(ile);
		fram.fireSignificantEditEvent();
	}

	/**
	 * Handles basic mouse method.
	 */
	public void mousePressed(MouseEvent evt)
	{
		if(evt.getModifiers()==MouseEvent.BUTTON3_MASK)
			delNeuronAt(evt.getPoint().y);
		else 
			markNeuronAt(evt.getPoint().y);
		repaint(0);		
	}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseReleased(MouseEvent evt){}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseDragged(MouseEvent evt) {}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseClicked(MouseEvent evt) {}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseEntered(MouseEvent evt) {}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseExited(MouseEvent evt) {}
	/**
	 * Handles basic mouse method.
	 */
	public void mouseMoved(MouseEvent evt) {}

	protected class MyFramstickListener implements FramstickListener
	{
		public void significantEdit(FramstickEvent framEvent)
		{
			setZoomValue();
			repaint(0);
		}

		public void markChanged(FramstickEvent framEvent)
		{
			setZoomValue();
			repaint(0);
		}
	}
}