/**
 * <p>Title: Fred v.2.0 - Framstick.java</p>
 * @author Cezary Dowhan
 * @version 2.0
 */

package jGDK;

import java.lang.ref.WeakReference;
import java.util.*;

public class Framstick
{
  static public Library library;
  
  public String name;
  public String info;
  public String author;
  public String genotype;
  
  private Vector undoBuffer;
  private Vector redoBuffer;
  private Model model;
  
  private FramstickEvent framEvent;
  private Vector listenerList;
  
  /**
   *
   */
  
  public Framstick()
  {
    init();
  }
  
  /**
   *
   */
  
  public Framstick(Library library)
  {
    Framstick.library = library;
    init();
  }
  
  /**
   *
   */
  
  public static void setLibrary(Library library)
  {
    Framstick.library = library;
  }
  
  /**
   *
   */
  
  private void init()
  {
    this.name = new String("Noname");
    this.info = new String("None");
    this.author = new String("Unknown");
    this.genotype = new String("//0\np:\n");
    
    undoBuffer = new Vector();
    redoBuffer = new Vector();
    model = new Model();
    listenerList = new Vector();
  }
  
  /**
   *
   */
  
  public void addFramstickListener(FramstickListener framListener)
  {
    synchronized (FramstickListener.class)
    {
      //System.out.println("adding listener:" + framListener);
      WeakReference wr = new WeakReference(framListener);
      listenerList.add(wr);
    }
  }
  
  /**
   *
   */
  
  public void removeFramstickListener(FramstickListener framListener)
  {
    synchronized (FramstickListener.class)
    {
      //System.out.println("removing listener:" + framListener);
      int size = listenerList.size();
      int i = 0;
      while (i < size)
      {
        WeakReference wr = (WeakReference)listenerList.elementAt(i);
        FramstickListener fl = (FramstickListener)wr.get();
        if (fl == null)
        {
          listenerList.removeElement(wr);
          size--;
        }
        else
        {
          if (fl == framListener)
            listenerList.removeElement(wr);
          i++;
        }
      }
    }
  }
  
  /**
   *
   */
  
  public void fireSignificantEditEvent()
  {
    synchronized (Framstick.class)
    {
      FramstickEvent e = null;
      int size = listenerList.size();
      int i = 0;
      //System.out.println("listeners=" + size);
      while (i < size)
      {
        WeakReference wr;
        try
        {
          wr = (WeakReference)listenerList.elementAt(i);
        }
        catch (ArrayIndexOutOfBoundsException e1)
        {
          return;
        }
        FramstickListener fl = (FramstickListener)wr.get();
        if (fl == null)
        {
          listenerList.removeElement(wr);
          size--;
          System.err.println("WeakRefFramstickListener.fireElementAdded: found dead reference:");
/*					for(int j=0;j<listenerList.size();j++)
                                                if(((WeakReference)listenerList.elementAt(j)).get()!=null)
                                                System.err.println(((WeakReference)listenerList.elementAt(j)).get());
 */				}
        else
        {
          if (e == null) // lazily create event
            e = new FramstickEvent(this);
          fl.significantEdit(e);
        }
        i++;
        //size = listenerList.size();
      }
    }
  }
  
  public void fireMarkChangedEvent()
  {
    FramstickEvent e = null;
    int size = listenerList.size();
    int i = 0;
    
    while (i < size)
    {
      WeakReference wr;
      try
      {
        wr = (WeakReference)listenerList.elementAt(i);
      }
      catch (ArrayIndexOutOfBoundsException e1)
      {
        return;
      }
      FramstickListener fl = (FramstickListener)wr.get();
      if (fl == null)
      {
        listenerList.removeElement(wr);
        size--;
        System.err.println("WeakRefFramstickListener.fireElementAdded: found dead reference");
/*				for(int j=0;j<listenerList.size();j++)
                                                if(((WeakReference)listenerList.elementAt(j)).get()!=null)
                                                System.err.println(((WeakReference)listenerList.elementAt(j)).get());
 */			}
      else
      {
        if (e == null) // lazily create event
          e = new FramstickEvent(this);
        fl.markChanged(e);
        i++;
      }
    }
  }
  
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public void setName(String name)
  {
    this.name = name;
  }
  
  /**
   *
   */
  
  public void setAuthor(String author)
  {
    this.author = author;
  }
  
  /**
   *
   */
  
  public void setInfo(String info)
  {
    this.info = info;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public String getName()
  {
    return name;
  }
  
  /**
   *
   */
  
  public String getAuthor()
  {
    return author;
  }
  
  /**
   *
   */
  
  public String getInfo()
  {
    return info;
  }
  
  /**
   *
   */
  
  public String getGenotype()
  {
    return modelToString();
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public int getPartCount()
  {
    return model.vParts.size();
  }
  
  /**
   *
   */
  
  public int getJointCount()
  {
    return model.vJoints.size();
  }
  
  /**
   *
   */
  
  public int getConnCount()
  {
    return model.vConnections.size();
  }
  
  /**
   *
   */
  
  public int getNeuroCount()
  {
    return model.vNeurons.size();
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean markPart(int partNo)
  {
    if (getPartCount() > partNo)
    {
      model.markPart(partNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean markJoint(int jointNo)
  {
    if (getJointCount() > jointNo)
    {
      model.markJoint(jointNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean markNeuron(int neuronNo)
  {
    if (getNeuroCount() > neuronNo)
    {
      model.markNeuron(neuronNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean markConnection(int connectionNo)
  {
    if (getConnCount() > connectionNo)
    {
      model.markConnection(connectionNo);
      return true;
    }
    return false;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean unmarkPart(int partNo)
  {
    if (getPartCount() > partNo)
    {
      model.unmarkPart(partNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean unmarkJoint(int jointNo)
  {
    if (getJointCount() > jointNo)
    {
      model.unmarkJoint(jointNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean unmarkNeuron(int neuronNo)
  {
    if (getNeuroCount() > neuronNo)
    {
      model.unmarkNeuron(neuronNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean unmarkConnection(int connectionNo)
  {
    if (getConnCount() > connectionNo)
    {
      model.unmarkConnection(connectionNo);
      return true;
    }
    return false;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean removePart(int partNo)
  {
    if (getPartCount() >= partNo)
    {
      redoBuffer.clear();
      undoBuffer.add(new Model(model));
      if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
      model.removePart(partNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean removeJoint(int jointNo)
  {
    if (getJointCount() >= jointNo)
    {
      redoBuffer.clear();
      undoBuffer.add(new Model(model));
      if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
      model.removeJoint(jointNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean removeNeuron(int neuroNo)
  {
    if (getNeuroCount() >= neuroNo)
    {
      redoBuffer.clear();
      undoBuffer.add(new Model(model));
      if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
      model.removeNeuron(neuroNo);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean removeConnection(int connNo)
  {
    if (getConnCount() >= connNo)
    {
      redoBuffer.clear();
      undoBuffer.add(new Model(model));
      if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
      model.removeConnection(connNo);
      return true;
    }
    return false;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean redo()
  {
    if (redoBuffer.size() > 0)
    {
      undoBuffer.add(model);
      model = (Model) redoBuffer.remove(redoBuffer.size()-1);
      return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean undo()
  {
    if (undoBuffer.size() > 0)
    {
      redoBuffer.add(model);
      model = (Model) undoBuffer.remove(undoBuffer.size()-1);
      return true;
    }
    return false;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean addPart(float x, float y, float z)
  {
    redoBuffer.clear();
    undoBuffer.add(new Model(model));
    if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
    
    Hashtable properties = new Hashtable();
    properties.put("x", new Float(x));
    properties.put("y", new Float(y));
    properties.put("z", new Float(z));
    
    Vector declared = new Vector();
    declared.add("x");
    declared.add("y");
    declared.add("z");
    properties.put("declared", declared);
    
    return model.addPart(properties);
  }
  
  /**
   *
   */
  
  public boolean addJoint(int partNo1, int partNo2)
  {
    if (getPartCount() > partNo1 && getPartCount() > partNo2)
    {
      
      redoBuffer.clear();
      undoBuffer.add(new Model(model));
      if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
      
      Hashtable properties = new Hashtable();
      properties.put("p1", new Integer(partNo1));
      properties.put("p2", new Integer(partNo2));
      
      Vector declared = new Vector();
      declared.add("p1");
      declared.add("p2");
      properties.put("declared", declared);
      
      return model.addJoint(properties);
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean addConnection(int n, int i)
  {
    redoBuffer.clear();
    undoBuffer.add(new Model(model));
    if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
    
    Hashtable properties = new Hashtable();
    Vector declared = new Vector();
    declared.add("n");
    declared.add("i");
    properties.put("declared", declared);
    
    properties.put("n", new Integer(n));
    properties.put("i", new Integer(i));
    
    return model.addConnection(properties);
  }
  
  /**
   *
   */
  
  public boolean addNeuron(String type)
  {
    redoBuffer.clear();
    undoBuffer.add(new Model(model));
    if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
    
    Hashtable properties = new Hashtable();
    properties.put("d", new String(type));
    
    Vector declared = new Vector();
    declared.add("d");
    properties.put("declared", declared);
    
    return model.addNeuron(properties);
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public Object getPartProp(int partNo, String param)
  {
    if (getPartCount() > partNo)
    {
      Part part = (Part) model.vParts.elementAt(partNo);
      if (part.properties.containsKey(param))
        return part.properties.get(param);
      else
      {
        Record record = library.getRecord("class", "p");
        return record.getParamProperty(param, "def");
      }
    }
    return null;
  }
  
  /**
   *
   */
  
  public Object getJointProp(int jointNo, String param)
  {
    if (getJointCount() > jointNo)
    {
      Joint joint = (Joint) model.vJoints.elementAt(jointNo);
      if (joint.properties.containsKey(param))
        return joint.properties.get(param);
      else
      {
        Record record = library.getRecord("class", "j");
        return record.getParamProperty(param, "def");
      }
    }
    return null;
  }
  
  /**
   *
   */
  
  public Object getConnProp(int connNo, String param)
  {
    if (getConnCount() > connNo)
    {
      Connection conn = (Connection) model.vConnections.elementAt(connNo);
      if (conn.properties.containsKey(param))
        return conn.properties.get(param);
      else
      {
        Record record = library.getRecord("class", "c");
        return record.getParamProperty(param, "def");
      }
    }
    return null;
  }
  
  /**
   *
   */
  
  public Object getNeuroProp(int neuroNo, String param)
  {
    if (getNeuroCount() > neuroNo)
    {
      Neuron neuron = (Neuron) model.vNeurons.elementAt(neuroNo);
      if (neuron.properties.containsKey(param))
        return neuron.properties.get(param);
      else
      {
        if (param.compareTo("info")==0)
        {
          neuron.properties.put("info",new Hashtable());
          ((Hashtable)neuron.properties.get("info")).put("declared",new Vector());
//          System.out.println("teraz stworzylem:"+((Hashtable)neuron.properties.get("info")));
        }
        Record record = library.getRecord("class", "n");
        return record.getParamProperty(param, "def");
      }
    }
    return null;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public void setPartProp(int partNo, String name, Object value)
  {
    if (getPartCount() > partNo)
    {
      Part part = (Part) model.vParts.get(partNo);
      part.properties.put(name, value);
      Vector declared = (Vector) part.properties.get("declared");
      if (declared.contains(name)) return;
      declared.add(name);
    }
  }
  
  /**
   *
   */
  
  public void setJointProp(int jointNo, String name, Object value)
  {
    if (getJointCount() > jointNo)
    {
      Joint joint = (Joint) model.vJoints.get(jointNo);
      if (name.equals("p1") || name.equals("p2"))
      {
        int integer = ((Integer) value).intValue();
        if (integer<0 || integer >= getPartCount())
        {
          ErrManager.send("Framstick", ErrManager.ER_ERROR, I18N.getMessage("incorrectValue"));
          return;
        }
      }
      
      if (name.equals("dx") || name.equals("dy") || name.equals("dz"))
      {
        float number = ((Float) value).floatValue();
        if (number<-2 || number>2)
        {
          ErrManager.send("Framstick", ErrManager.ER_ERROR, I18N.getMessage("incorrectValue"));
          return;
        }
      }
      
      joint.properties.put(name, value);
      Vector declared = (Vector) joint.properties.get("declared");
      if (declared.contains(name)) return;
      declared.add(name);
    }
  }
  
  /**
   *
   */
  
  public void setNeuroProp(int neuroNo, String name, Object value)
  {
    if (getNeuroCount() > neuroNo)
    {
      Neuron neuron = (Neuron) model.vNeurons.get(neuroNo);
      if (name.equals("p"))
      {
        int integer = ((Integer) value).intValue();
        if (integer<0 || integer >= getPartCount())
        {
          ErrManager.send("Framstick", ErrManager.ER_ERROR, I18N.getMessage("incorrectValue"));
          return;
        }
      }
      
      if (name.equals("j"))
      {
        int integer = ((Integer) value).intValue();
        if (integer<0 || integer >= getJointCount())
        {
          ErrManager.send("Framstick", ErrManager.ER_ERROR, I18N.getMessage("incorrectValue"));
          return;
        }
      }
      
      neuron.properties.put(name, value);
      Vector declared = (Vector) neuron.properties.get("declared");
      if (declared.contains(name)) return;
      declared.add(name);
    }
  }
  
  /**
   *
   */
  
  public void setConnProp(int connNo, String name, Object value)
  {
    if (getConnCount() > connNo)
    {
      Connection conn = (Connection) model.vConnections.get(connNo);
      conn.properties.put(name, value);
      Vector declared = (Vector) conn.properties.get("declared");
      if (declared.contains(name)) return;
      declared.add(name);
    }
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public void delPartProp(int partNo, String name)
  {
    Part part = (Part) model.vParts.elementAt(partNo);
    part.properties.remove(name);
    Vector declared = (Vector) part.properties.get("declared");
    if (declared.contains(name)) declared.remove(name);
    
  }
  
  /**
   *
   */
  
  public void delJointProp(int jointNo, String name)
  {
    Joint joint = (Joint) model.vJoints.elementAt(jointNo);
    joint.properties.remove(name);
    Vector declared = (Vector) joint.properties.get("declared");
    if (declared.contains(name)) declared.remove(name);
    
  }
  
  /**
   *
   */
  
  public void delConnProp(int connNo, String name)
  {
    Connection conn = (Connection) model.vConnections.elementAt(connNo);
    conn.properties.remove(name);
    Vector declared = (Vector) conn.properties.get("declared");
    if (declared.contains(name)) declared.remove(name);
  }
  
  /**
   *
   */
  
  public void delNeuroProp(int neuroNo, String name)
  {
    Neuron neuro = (Neuron) model.vNeurons.elementAt(neuroNo);
    neuro.properties.remove(name);
    Vector declared = (Vector) neuro.properties.get("declared");
    if (declared.contains(name)) declared.remove(name);
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean partPropExists(int partNo, String name)
  {
    if (getPartCount() > partNo)
    {
      Part part = (Part) model.vParts.get(partNo);
      if (part.properties.containsKey(name))
        return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean jointPropExists(int jointNo, String name)
  {
    if (getJointCount() > jointNo)
    {
      Joint joint = (Joint) model.vJoints.get(jointNo);
      if (joint.properties.containsKey(name))
        return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean connPropExists(int connNo, String name)
  {
    if (getConnCount() > connNo)
    {
      Connection connection = (Connection) model.vConnections.get(connNo);
      if (connection.properties.containsKey(name))
        return true;
    }
    return false;
  }
  
  /**
   *
   */
  
  public boolean neuroPropExists(int neuroNo, String name)
  {
    if (getNeuroCount() > neuroNo)
    {
      Neuron neuron = (Neuron) model.vNeurons.get(neuroNo);
      if (neuron.properties.containsKey(name))
        return true;
    }
    return false;
  }
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public void translatePartCoords(int partNo, float mx, float my, float mz)
  {
    if (getPartCount() > partNo)
    {
      Part part = (Part) model.vParts.elementAt(partNo);
      float value;
      
      
      if (part.properties.containsKey("x"))
      {
        value = ((Float)part.properties.get("x")).floatValue();
        part.properties.put("x",new Float(value+mx));
      }
      else part.properties.put("x",new Float(mx));
      
      
      if (part.properties.containsKey("y"))
      {
        value = ((Float)part.properties.get("y")).floatValue();
        part.properties.put("y",new Float(value+my));
      }
      else part.properties.put("y",new Float(mx));
      
      
      if (part.properties.containsKey("z"))
      {
        value = ((Float)part.properties.get("z")).floatValue();
        part.properties.put("z",new Float(value+mz));
      }
      else part.properties.put("z",new Float(mz));
      
      for (int i=0;i<model.vJoints.size();i++)
      {
        int p2=((Integer)((Joint)model.vJoints.elementAt(i)).properties.get("p2")).intValue();
        float dxTest=0;
        if (((Hashtable)((Joint)model.vJoints.elementAt(i)).properties).containsKey("dx")) dxTest=((Float)((Hashtable)((Joint)model.vJoints.elementAt(i)).properties).get("dx")).floatValue();
        if (dxTest!=0&&p2==partNo)
        {
          ((Hashtable)((Joint)model.vJoints.elementAt(i)).properties).put("dx",new Float(0.0));
          Object[] inon=
          {""+i};
          /*M*/	ErrManager.send( ErrManager.ER_WARNING, I18N.getFormated("framBroken",inon));
          
          
        }
        
      }
    }
  }
  
  
  /**
   *
   */
  
  public void recountCoordinates()
  {
    model.recountCo();
  }
  
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean elementStrongSelected(String type,int nr)
  {
    //sprawdzic czy nr nie wybiega poza tablice elementow danego typu
    
    if (type.equals("j")) return ((Element)model.vJoints.elementAt(nr)).stSelected;
    if (type.equals("p")) return ((Element)model.vParts.elementAt(nr)).stSelected;
    if (type.equals("n")) return ((Element)model.vNeurons.elementAt(nr)).stSelected;
    if (type.equals("c")) return ((Element)model.vConnections.elementAt(nr)).stSelected;
    return false;
    
  }
  
  /**
   *
   */
  
  public boolean elementWeakSelected(String type,int nr)
  {
    //sprawdzic czy nr nie wybiega poza tablice elementow danego typu
    
    if (type.equals("p")) return ((Element)model.vParts.elementAt(nr)).weSelected;
    if (type.equals("j")) return ((Element)model.vJoints.elementAt(nr)).weSelected;
    if (type.equals("n")) return ((Element)model.vNeurons.elementAt(nr)).weSelected;
    if (type.equals("c")) return ((Element)model.vConnections.elementAt(nr)).weSelected;
    return false;
  }
  
  // the end of group methods ------------------------------------------------------
  
  /**
   *
   */
  
  public boolean rebuildModel()
  {
    GenotypeParser parser = new GenotypeParser("class", genotype);
    
    Model newModel = parser.createModel();
    
    if (newModel != null)
    {
      model = newModel;
      checkValidity();
      
      return true;
    }
    
    return false;
  }
  // the end of method: rebuildModel() -----------------------------------------------
  
  public void checkValidity() //added - mac
  {
    if (getJointCount()==0 && getPartCount()<2) return; //don't check empty models
    Record record = library.getRecord("class", "j");
    float maxdist=((Float)(record.getParamProperty("dx", "max"))).floatValue();
    
    HashSet s=new HashSet(); //all connected parts
    for(int i=0;i<getJointCount();i++)
    {
      Integer i1=(Integer)getJointProp(i,"p1");
      Integer i2=(Integer)getJointProp(i,"p2");
      int p1=(i1).intValue();
      int p2=(i2).intValue();
      s.add(i1);
      s.add(i2);
      //should be a method for this elsewhere!:
      float x1 = ((Float)getPartProp(p1,"x")).floatValue();//wsp. 3D laczonych partow
      float x2 = ((Float)getPartProp(p2,"x")).floatValue();
      float y1 = ((Float)getPartProp(p1,"y")).floatValue();
      float y2 = ((Float)getPartProp(p2,"y")).floatValue();
      float z1 = ((Float)getPartProp(p1,"z")).floatValue();
      float z2 = ((Float)getPartProp(p2,"z")).floatValue();
      
      float dist=(float)Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
      if (dist>maxdist)
      {
        Object[] args=
        {""+i,""+dist,""+maxdist};
        ErrManager.send("Framstick", ErrManager.ER_ERROR,I18N.getFormated("Jtoolong",args));
      }
    }
    for(int i=0;i<getPartCount();i++)
      if (!s.contains(new Integer(i)))
      {
        Object[] args=
        {""+i};
        ErrManager.send("Framstick", ErrManager.ER_ERROR,I18N.getFormated("Pseparated",args));
      }
  }
  
  public String modelToString()
  {
    StringBuffer buffer = new StringBuffer();
    Record record;
    
    buffer.append("//0\n");
    
    for (int i=0; i<getPartCount(); i++)
    {
      Part part = (Part) model.vParts.get(i);
      record = library.getRecord("class", "p");
      buffer.append("p: "+elementToString(record, part.properties)+"\n");
    }
    
    for (int i=0; i<getJointCount(); i++)
    {
      Joint joint = (Joint) model.vJoints.get(i);
      record = library.getRecord("class", "j");
      buffer.append("j: "+elementToString(record, joint.properties)+"\n");
    }
    
    for (int i=0; i<getNeuroCount(); i++)
    {
      Neuron neuron = (Neuron) model.vNeurons.get(i);
      record = library.getRecord("class", "n");
      Hashtable auxHashtable = new Hashtable(neuron.properties);
      String dField = (String) auxHashtable.remove("d");
      String nSection = elementToString(record, auxHashtable);
      buffer.append("n: "+nSection);
      if (dField != null)
      {
        if (!nSection.equals("")) buffer.append(", ");
        buffer.append("d=\""+dField);
        auxHashtable = (Hashtable) auxHashtable.get("info");
        if (auxHashtable != null)
        {
          record = library.getRecord("neuro",dField);
          buffer.append(": "+elementToString(record, auxHashtable));
        }
        buffer.append('"');
      }
      buffer.append('\n');
    }
    
    for (int i=0; i<getConnCount(); i++)
    {
      Connection conn = (Connection) model.vConnections.get(i);
      record = library.getRecord("class", "c");
      buffer.append("c: "+elementToString(record, conn.properties)+"\n");
    }
    
    return buffer.toString();
  }
  
/*
        private String commentChars(String string)
        {
                if (string.indexOf('~') != -1  || string.indexOf('"') != -1) {
 
 
                        return ;
                }
 
                return string;
        }
 */
  
  // the end of method: modelToString() ----------------------------------------------
  
  /**
   *
   */
  
  private String elementToString(Record record, Hashtable properties)
  {
    StringBuffer buffer = new StringBuffer();
    
    Vector declared	= (Vector) properties.get("declared");
    String param;
    
    boolean skipped = false;
    int position = 0;
    
    while ((param=record.getParamAtPos(position)) != null)
    {
      position++;
      Integer flags = (Integer) record.getParamProperty(param,"flags");
      if (properties.containsKey(param))
      {
        if (!declared.contains(param))
        {
          skipped = true;
          continue;
        }
        Object value = properties.get(param);
        if (flags == null || flags.intValue() == 1024)
        {
          if (!skipped)
          {
            buffer.append(value.toString()+", ");
            continue;
          }
        }
        buffer.append(param.toString()+"="+value.toString()+", ");
        skipped = false;
      }
      else skipped = true;
    }
    int lastComma = buffer.length()-2;
    if (lastComma > 0) buffer.setCharAt(lastComma, ' ');
    
    return buffer.toString().trim();
  }
  // the end of method: elementToString() --------------------------------------------
  
  /**
   *
   */
  
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    
    buffer.append("org:\n");
    buffer.append("name:"+name+"\n");
    buffer.append("info:~\n"+info+"\nCREATED BY: "+author+"~\n");
    buffer.append("genotype:~\n"+modelToString()+"~\n");
    
    return buffer.toString();
  }
}


