package jGDK;

import java.util.Hashtable;
import java.util.Vector;

/**
 * <p>Title: Fred v.2.0 - GenotypeParser.java</p>
 * @author Cezary Dowhan
 * @version 2.0
 */

public class GenotypeParser extends F0Automaton
{
  static final int GP_EOF		= 0;
  static final int GP_ERROR	= 1;
  static final int GP_EOL		= 2;
  
  public static Library library;
  private Record record;
  
  public String		sort;
  public String		classId;
  public Hashtable	properties;
  
  public Vector	declared;//New
  
  private String	variable;
  private String	type;
  private Object	value;
  private int		position;
  
  public GenotypeParser(String sort, String genotype)
  {
    super(genotype);
    properties	= new Hashtable();
    declared	= new Vector();//New
    this.sort = sort;
  }
  // the end of the initial section -------------------------------------------------
  
  /**	Okresla format genotypu	*/
  
  public int determineFormat()
  {
    if (nextToken() != ID_FORMAT)
    {
      parseError(ErrManager.ER_ERROR, "Expected format definition");
      return -1;
    }
    
    Integer number = parseInt(tokenMaster.data);
    if (number == null)
    {
      parseError(ErrManager.ER_ERROR, "Number expected in the Format section");
      return -1;
    }
    
    return number.intValue();
  }
  // the end of public method: determineFormat() ------------------------------------
  
  /**	Pobiera i dekoduje pojedyncza linie genotypu */
  
  public int nextLine()
  {
    properties.clear();
    declared = new Vector();	//New
    classId	 = null;
    position = 0;
    
    do if (nextToken() == ID_EOF) return GP_EOF;
    while (tokenMaster.type == ID_EOL);
    
    // header
    
    if (tokenMaster.type != ID_CLASS)
    {
      parseError(ErrManager.ER_ERROR, "Expected class definition");
      return GP_ERROR;
    }
    
    classId = (String) tokenMaster.data;
    record	= library.getRecord(sort, classId);
    
    if (record == null)
    {
      parseError(ErrManager.ER_ERROR, classId+": - unknown class");
      return GP_ERROR;
    }
    // parsing class body
    
    properties.put("declared", declared);
    
    while (nextToken() != ID_EOL && tokenMaster.type != ID_EOF)
    {
      if (tokenMaster.type == ID_ERROR || tokenSlave.type == ID_ERROR)
      {
        parseError(ErrManager.ER_ERROR, "Syntax error");
        skipLine();
        return GP_EOL;
      }
      
      switch (tokenMaster.type)
      {
        case ID_VARIABLE:
          variable	= (String) tokenMaster.data;
          value		= tokenSlave.data;
          position	= record.getPosOfParam((String) variable);
          
          if (position == -1)
          {
            parseError(ErrManager.ER_ERROR, "Variable "+variable+" - unknown parameter");
            skipLine();
            return GP_EOL;
          }
          if (checkConstraints(tokenSlave) == false)
          {
            skipLine();
            return GP_EOL;
          }
          
          declared.add(variable);
          properties.put(variable, value); // if everything is ok
          break;
          //------------------------------------------------------------------
        case ID_WORD:
        case ID_QUOTE:
          variable	= record.getParamAtPos(position+tokenMaster.offset);
          value		= tokenMaster.data;
          
          if (variable == null)
          {
            parseError(ErrManager.ER_ERROR, "Too many parameters");
            skipLine();
            return GP_EOL;
          }
          // checking properties constraints
          Integer flags = (Integer) record.getParamProperty(variable, "flags");
          if (flags != null && flags.intValue() == 0)
            parseError(ErrManager.ER_WARNING, "Variable "+variable+" should be determined directly");
          
          if (checkConstraints(tokenMaster) == false)
          {
            skipLine();
            return GP_EOL;
          }
          
          declared.add(variable);
          properties.put(variable, value); // if everything is ok
          break;
          //------------------------------------------------------------------
        default:
          parseError(ErrManager.ER_ERROR, "Unknown error 2");
          skipLine();
          return GP_EOL;
      }	// the end of switch section
    };		// the end of while section
    
    return GP_EOL;
  }
  // the end of public method: nextLine() -------------------------------------------
  
  /**	Sprawdza ograniczenia nalozone na kazda ze zmiennych */
  
  private boolean checkConstraints(Token dataToken)
  {
    type = (String) record.getParamProperty(variable, "type");
    //-----------------------------------------------
    if (type.equals("s"))
    {
      if (dataToken.type != ID_QUOTE &&
      dataToken.type != ID_WORD)
      {
        parseError(ErrManager.ER_ERROR, "Variable "+variable+". Incompatible types. Expected string");
        return false;
      };
    }
    //-----------------------------------------------
    if (type.equals("f"))
    {
      Float number = parseFloat(dataToken.data);
      
      if (number == null)
      {
        parseError(ErrManager.ER_ERROR, "Variable "+variable+". Incompatible types. Expected a number");
        return false;
      };
      
      Float min = (Float) record.getParamProperty(variable, "min");
      Float max = (Float) record.getParamProperty(variable, "max");
      Float def = (Float) record.getParamProperty(variable, "def");
      
      if (min != null)
      {
        if (number.floatValue() < min.floatValue())
        {
          parseError(ErrManager.ER_ERROR, "Variable "+variable+". Out of bounds");
          return false;
        };
      };
      if (max != null)
      {
        if (number.floatValue() > max.floatValue())
        {
          parseError(ErrManager.ER_ERROR, "Variable "+variable+". Out of bounds");
          return false;
        };
      };
      
      value = number;
    }
    //-----------------------------------------------
    if (type.equals("d"))
    {
      Integer number = parseInt(dataToken.data);
      
      if (number == null)
      {
        parseError(ErrManager.ER_ERROR, "Variable "+variable+". Incompatible types. Expected a number");
        return false;
      };
      
      Integer min = (Integer) record.getParamProperty(variable, "min");
      Integer max = (Integer) record.getParamProperty(variable, "max");
      Integer def = (Integer) record.getParamProperty(variable, "def");
      
      if (min != null)
      {
        if (number.intValue() < min.intValue())
        {
          parseError(ErrManager.ER_ERROR,  "Variable "+variable+". Out of bounds");
          return false;
        };
      };
      if (max != null)
      {
        if (number.intValue() > max.intValue())
        {
          parseError(ErrManager.ER_ERROR,  "Variable "+variable+". Out of bounds");
          return false;
        };
      };
      
      value = number;
    }
    //-----------------------------------------------
    return true;
  }
  // the end of private method: checkConstraints() ----------------------------------
  
  /**	Dodaje odpowiedni naglowek do komunikatow */
  
  private void parseError(int errCode, String message)
  {
    String announcement = "";
    
    switch (errCode)
    {
      case ErrManager.ER_ERROR:
        announcement = "Error in line "+lineNo+". ";
        break;
      case ErrManager.ER_WARNING:
        announcement = "Warning in line "+lineNo+". ";
    }
    
    ErrManager.send(errCode, announcement+message);
  }
  // the end of private method: parseError() ----------------------------------------
  
  public Model createModel()
  {
    int format = determineFormat();
    
    if (format == -1) format=1;
    if (format > 0)
    {
      ErrManager.send(ErrManager.ER_ERROR, "Format f"+format+" is not supported");
      return null;
    }
    
    Model model = new Model();
    
    while (nextLine() == GenotypeParser.GP_EOL)
    {
      if (classId.equals("m")) //'model' (rarely used) - fred ignores this line.
      {
        parseError(ErrManager.ER_WARNING, "line 'm:' ignored");
        continue;
      }
      
      if (classId.equals("p"))
      {
        model.addPart(properties);
        continue;
      }
      
      if (classId.equals("j"))
      {
        model.addJoint(properties);
        continue;
      }
      
      if (classId.equals("c"))
      {
        model.addConnection(properties);
        continue;
      }
      
      if (classId.equals("n"))
      {
        String dString = (String) properties.get("d");
        if (dString != null)
        {
          F0Automaton d = new F0Automaton(dString);
          switch (d.nextToken())
          {
            case F0Automaton.ID_CLASS: // for example d="N:0.3,fo=3"
              GenotypeParser dField = new GenotypeParser("neuro", dString);
              dField.lineNo = lineNo-1;
              if (dField.nextLine() == GenotypeParser.GP_ERROR)
              {
                properties.put("d", "N");
                model.addNeuron(properties);
                continue;
              }
              properties.put("d", dField.classId);
	      
	      // tu bylo "properties" zamiast "dField.properties"
	      // w efekcie parametry w "d" znikaly (nikt tego nie zauwazyl?!)
              properties.put("info", dField.properties);
		
              model.addNeuron(properties);
              continue;
            case F0Automaton.ID_WORD: // for example d="T "
              String classId = (String) d.tokenMaster.data;
              Record record = library.getRecord("neuro", classId);
              if (record == null)
              {
                ErrManager.send(ErrManager.ER_ERROR, "Error in line "+(lineNo-1)+". Unknown class in field d");
                properties.put("d", "N");
                model.addNeuron(properties);
                continue;
              }
              properties.put("d", classId);
              model.addNeuron(properties);
              continue;
            default:
              ErrManager.send(ErrManager.ER_ERROR, "Error in line "+lineNo+". Unknown class2");
          }
        }
        properties.put("d", "N");
        model.addNeuron(properties);
      }
    }
    return model;
  }
  
  
};