package jGDK;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.*;

/**
 * <p>Title: Fred v.2.0 - Library.java</p>
 * @author Cezary Dowhan
 * @version 2.0
 */

public final class Library extends F0Automaton
{
  final String intVariables	= "flags|xtra|inputs|output|location|vhints";
  
  public  Vector		database;
  
  private	Hashtable	param;
  private String		name;
  private String		variable;
  private Object		amount;
  private	Record		record;
  
  
  public static String load(java.net.URL url) throws IOException
  {
      java.net.URLConnection uc=url.openConnection();
      java.io.InputStream is=uc.getInputStream();
      StringBuffer sb=new StringBuffer();
      while(is.available()>0)
	{
	byte buf[] = new byte[is.available()];
	int n=is.read(buf);
	if (n<0) break;
	sb.append(new String(buf,0,n));
	}
      is.close();
      String s=sb.toString();
      return s;
  }
  
  // constructor ----------------------------------------------------------
  
  protected void addTokenizerSettings()	// Adapter
  {
    st.commentChar('#'); //additional StreamTokenizer setting
  }
  
  // ----------------------------------------------------------------------
  /**	Create database using input file */
  
  public Library(String fileContents)
  {
    super(fileContents);
    try
    {
      database = new Vector();
      createLibrary();
    }
    catch (IOException ioEx)
    {
      ErrManager.send(ErrManager.ER_ERROR, "Failed to initialize library");
      database = null;
    }
  }

  // tworzy library i od razu wczytuje standardowy model.def
  public Library() throws Exception
  {
  this(load(Library.class.getResource("model.def")));
  }
  
  //the end of initial section --------------------------------------
  
  /**	Get wanted record from database	 */
  
  public Record getRecord(String sort, String id)
  {
    for (int index=0; index<database.size(); index++)
    {
      Record record = (Record) database.get(index);
      if (record.sort.equals(sort))
      {
        String key = (String) record.info.get("id");
        if (key != null && key.equals(id))
          return record;
      }
    }
    return null;
  }
  //-----------------------------------------------------------------
  
  public Record[] getAllRecords(String sort) ///returns all records with their names as "id (name)"
  {
    List l=new ArrayList();
    for (int index=0; index<database.size(); index++)
    {
      Record record = (Record) database.get(index);
      if (record.sort.equals(sort)) l.add(record);
    }
    return (Record[])(l.toArray(new Record[0]));
  }
  //-----------------------------------------------------------------

  /**	Search input file for genotype language definitions and create database	 */
  
  private void createLibrary() throws IOException
  {
    while (nextToken() != ID_EOF)
    {
      if (tokenMaster.type == ID_EOL)
        continue;
      if (tokenMaster.type == ID_CLASS)
      {
        param	= new Hashtable();
        name	= (String) tokenMaster.data;
        
        if (name.equals("class") || name.equals("neuro"))
        {
          record = new Record();
          database.addElement(record);
          record.addInfo(param);
          record.sort = name;
        }
        if (name.equals("prop"))
        {
          record.addParameter(param);
        }
        
        // read the rest of current line
        
        while (nextToken() != ID_EOL)
        {
          if (tokenMaster.type == ID_VARIABLE)
          {
            variable = (String) tokenMaster.data;
            amount   =	tokenSlave.data;
            //-------------------------------------
            if (variable.equals("symbol"))
            {
              StringTokenizer pt = new StringTokenizer((String) amount, ",");
              Vector symbol = new Vector();
              while (pt.hasMoreTokens())
	      {
                symbol.add(new Integer(pt.nextToken()));
	      }
              int[] flatSymbol = new int[symbol.size()];
              for (int i=0;i<symbol.size(); i++)
                flatSymbol[i]= ((Integer)symbol.get(i)).intValue();
              param.put("symbol", flatSymbol);
              continue;
            }
            //-------------------------------------
            if (variable.equals("type"))
            {
              StringTokenizer pt = new StringTokenizer((String) amount, " ");
              
              if (pt.hasMoreTokens())
              {
                String sType = pt.nextToken();
                param.put("type", sType);
                
                if (sType.equals("f"))
                {
                  if (pt.hasMoreTokens()) param.put("min", new Float(pt.nextToken()) );
                  if (pt.hasMoreTokens()) param.put("max", new Float(pt.nextToken()) );
                  if (pt.hasMoreTokens()) param.put("def", new Float(pt.nextToken()) );
                  continue;
                }
                
                if (sType.equals("d"))
                {
                  if (pt.hasMoreTokens()) param.put("min", new Integer(pt.nextToken()) );
                  if (pt.hasMoreTokens()) param.put("max", new Integer(pt.nextToken()) );
                  if (pt.hasMoreTokens()) param.put("def", new Integer(pt.nextToken()) );
                  continue;
                }
                
                if (sType.equals("s"))
                {
                  String lancuch;
                  if (!pt.hasMoreTokens()) continue;
                  lancuch = pt.nextToken();
                  if(lancuch.equals("0"))
                    param.put("min", new String(""));
                  else if (!lancuch.equals("-1"))
                    param.put("min", new String(lancuch));
                  
                  if (!pt.hasMoreTokens()) continue;
                  lancuch = pt.nextToken();
                  if(lancuch.equals("0"))
                    param.put("max", new String(""));
                  else if (!lancuch.equals("-1"))
                    param.put("max", new String(lancuch));
                  
                  if (!pt.hasMoreTokens()) continue;
                  lancuch = pt.nextToken();
                  if(lancuch.equals("0"))
                    param.put("def", new String(""));
                  else if (!lancuch.equals("-1"))
                    param.put("def", new String(lancuch));
                  continue;
                }
              }
            } // <--  if (variable.equals("type"))
            
            if (intVariables.indexOf(variable) != -1)
            {
              amount = parseInt((String) amount);
            }
            param.put(variable, amount);
            continue;
          } // <-- if (tokenMaster.type == ID_VARIABLE)
          ErrManager.send(ErrManager.ER_ERROR, "Expected variable=value");
          return;
        } // <-- while (nextToken("") != ID_EOL)
        continue;
      } // <-- if (tokenMaster.type == ID_CLASS)
      ErrManager.send(ErrManager.ER_ERROR, "Expected class definition [class: ... ]");
      return;
    } // <-- while (nextToken("") != ID_EOF)
  }
  
  //-----------------------------------------------------------------
  
  /**	Convert database contents to String */
  
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    
    for (int i = 0; i < database.size(); i++)
    {
      buffer.append("##############\n");
      buffer.append("  Record "+i+"\n");
      buffer.append("##############\n");
      buffer.append(database.get(i)+"\n");
    }
    
    return buffer.toString();
  }
  // ------------------------------------------------------------------------
  
  public static void main(String[] args)
  {
    Library library = new Library(readFile("genotype.config"));
    System.out.println(library);
  }
  
  private static String readFile(String filename)
  {
    StringBuffer fileContents = new StringBuffer();
    
    try
    {
      FileReader		file	= new FileReader(filename);
      BufferedReader  buffer  = new BufferedReader(file);
      
      int character;
      while ((character=buffer.read()) != -1)
        fileContents.append((char) character);
      
      return fileContents.toString();
      
    }
    catch (FileNotFoundException f)
    {
      System.err.println("Plik "+filename+" nie istnieje");
    }
    catch (IOException e)
    {
      System.err.println("Blad IO");
    }
    
    return null;
  }
  
}
